//@@@
//user:    GSC
//date:    25/01/2024
//ver:     4.6.0
//project: TEST 4.6.0
//type:    event trigger (TRIGGER DI CLASSE)
//class:   gwr_report
//note:    gwr_report event trigger
//@@@

import java.net.URL;
import org.springframework.core.env.Environment;
import com.geowebframework.transfer.objects.exception.*;
import com.geowebframework.dataservice.model.exportableReport.ExportableReport;
	

public class GwrReportEventTrigger extends com.geowebframework.dataservice.querybuilder.EventTrigger {
	
	private final String CLASS_NAME = "gwr_report";
	private final String CLASS_TABLE_NAME = CLASS_NAME;
	
	public boolean beforeInsert(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeInsert()");
		
		String radix = "EXP_REP_";
		String cod = "cod_report";
		Integer leadingZeros = 6;
		String leadedInt = String.format("%0"+leadingZeros+"d", 1);
		String query = "SELECT "+cod+" FROM "+CLASS_TABLE_NAME+" ORDER BY "+cod+" asc";
		def res = services.queryService.executeQuery(query, null);
		if(res.size()!=0){
			String cod_model_last = res[res.size()-1][cod];
			Integer lastInt = Integer.parseInt(cod_model_last.substring(cod_model_last.indexOf(radix)+radix.length()));
			lastInt++;
			leadedInt = String.format("%0"+leadingZeros+"d", lastInt);
		}
		String newCod = radix+leadedInt;
		valuesMap[cod] = newCod;
		
		return true;
	};
	
	public boolean afterInsert(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterInsert()");
		
		def cod_report = valuesMap.cod_report;
		def main_report_name = valuesMap.main_report_name;
		def report_resources = valuesMap.report_resources;
		
		def has_parameters_to_prompt = 0;
		if(cod_report!=null && main_report_name!=null && report_resources!=null){
			ExportableReport exportableReport = new ExportableReport();
			exportableReport.codReport = cod_report;
			exportableReport.mainReportName = main_report_name;
			exportableReport.reportResources = report_resources;
			
			List<Map<String,Object>> parametersToPromptList = services.gwExportableReportService.getParametersToPromptList(exportableReport);
			if(parametersToPromptList!=null && !parametersToPromptList.isEmpty()){
				has_parameters_to_prompt = 1;
				
				log.debug("parametersToPromptList - size: "+parametersToPromptList.size());
			}
		}
		
		String query = "UPDATE "+CLASS_TABLE_NAME+" SET has_parameters_to_prompt=#{map.has_parameters_to_prompt} WHERE cod_report=#{map.cod_report}";
		services.queryService.executeUpdateQuery(query, [cod_report: cod_report, has_parameters_to_prompt: has_parameters_to_prompt]);
		
		
		services.gwMetadataCacheService.clearAllStartingWith("gwExportableReportService_");
		
		return true;
	};
	
	public boolean beforeUpdate(HashMap<String,Object> valuesMap,HashMap<String,Object> oldValuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeUpdate()");
		
		
		return true;
	};
	
	public boolean afterUpdate(HashMap<String,Object> valuesMap,HashMap<String,Object> oldValuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterUpdate()");
		
		log.debug("valuesMap: ");
		log.debug(valuesMap.toString());
		
		
		def cod_report = valuesMap.cod_report;
		if(cod_report==null) cod_report = oldValuesMap.cod_report;
		log.debug("cod_report: "+cod_report);
		
		def main_report_name = valuesMap.main_report_name;
		if(main_report_name==null) main_report_name = oldValuesMap.main_report_name;
		log.debug("main_report_name: "+main_report_name);
		
		def report_resources = valuesMap.report_resources;
		if(report_resources==null) report_resources = oldValuesMap.report_resources;
		if(report_resources==null){
			String query = "SELECT report_resources FROM "+CLASS_TABLE_NAME+" WHERE cod_report=#{map.cod_report}";
			def report = services.queryService.executeQuery(query, [cod_report: cod_report])[0];
			if(report!=null) report_resources = report.report_resources;
		}
		log.debug("report_resources: "+report_resources);
		
		def has_parameters_to_prompt = 0;
		if(cod_report!=null && main_report_name!=null && report_resources!=null){
			ExportableReport exportableReport = new ExportableReport();
			exportableReport.codReport = cod_report;
			exportableReport.mainReportName = main_report_name;
			exportableReport.reportResources = report_resources;
			
			List<Map<String,Object>> parametersToPromptList = services.gwExportableReportService.getParametersToPromptList(exportableReport);
			if(parametersToPromptList!=null && !parametersToPromptList.isEmpty()){
				has_parameters_to_prompt = 1;
				
				log.debug("parametersToPromptList - size: "+parametersToPromptList.size());
			}
		}
		
		String query = "UPDATE "+CLASS_TABLE_NAME+" SET has_parameters_to_prompt=#{map.has_parameters_to_prompt} WHERE cod_report=#{map.cod_report}";
		services.queryService.executeUpdateQuery(query, [cod_report: cod_report, has_parameters_to_prompt: has_parameters_to_prompt]);
		
		
		services.gwMetadataCacheService.clearAllStartingWith("gwExportableReportService_");
		
		return true;
	};
	
	public boolean beforeDelete(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeDelete()");
		
		return true;
	};
	
	public boolean afterDelete(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterDelete()");
		
		services.gwMetadataCacheService.clearAllStartingWith("gwExportableReportService_");
		
		return true;
	};

} 